-- Multicinemes PR1 - Practica 4
-- Programadors: Sergio Blanco Cuaresma
--               Francesc Galiano Baixauli
-- Data: Maig 2001
-- Descripcio: Programa que porta el manteniment d'un cinema
-- *****************************************************************
with Ada.Text_IO; use Ada.Text_IO;
with multicinema; use multicinema;
with Ada.Numerics.Discrete_Random;
with Ada.Sequential_IO;
with AdaGraph; use AdaGraph;

procedure cine is

package enter_io is new ada.text_io.integer_io(integer); use enter_io;
package dia_IO is new Ada.Text_IO.Enumeration_IO(dia); use dia_IO;
package genere_IO is new Ada.Text_IO.Enumeration_IO(genere); use genere_IO;

type opcions is (a,b,c,d,e,f,g,h,i,j,s); -- Menu opcions
package opcions_IO is new Ada.Text_IO.Enumeration_IO(opcions);
use opcions_IO;

-- Fitxer sequencial (Historic)
type historictype is record
   pelicula :string (1..200):="null                                                                                                                                                                                                    ";
   espectadors: natural:=0;
   setmanes: natural:=0;
end record;
package s_io is new Ada.Sequential_IO(historictype); use s_io;
type historicarray is array (1..15) of historictype;


-- Numero aleatori
subtype sessioal is integer range 1..3;
package dr1 is new Ada.Numerics.Discrete_Random(sessioal); use dr1;
subtype salaal is integer range 1..15;
package dr2 is new Ada.Numerics.Discrete_Random(salaal); use dr2;
subtype butacaal is integer range 1..150;
package dr3 is new Ada.Numerics.Discrete_Random(butacaal); use dr3;

-- Variables Globals
cine: cinema;
lnom: integer;  -- Longitud del nom del cine (guardat a inicitalizar i utilitzat per mostrar el nom)
op: opcions:=a;
historic: historicarray; -- Aqui guardarem les pelicules que futurament seran historiques



--------------------------------------------------------------------------
-- Procediment per inicialitzar el cinema i crear el fitxerhistoric
--------------------------------------------------------------------------
procedure inicialitzar(fitxerhistoric: in string) is

cinenom: string (1..30);
cinesales: num_sales;
cinecapacitat: natural;
cine_diaesp: dia;
cinepreu, cinepreu_cs, cinepreu_esp: natural;
fitxerhist: s_io.file_type;

begin
   -- Crear el fitxer
   create(fitxerhist, out_file, fitxerhistoric);
   close(fitxerhist);
   -- Demanem les dades necessaries per crear el cinema
   put("Nom cinema: ");
   get_line(cinenom, lnom);
   put("Numero de sales: ");
   get(cinesales);
   put("Capacitat de les sales: ");
   get(cinecapacitat);
   put("Dia de l'espectador: ");
   get(cine_diaesp);
   put("Preu normal: ");
   get(cinepreu);
   put("Preu caps de setmana: ");
   get(cinepreu_cs);
   put("Preu dia de l'espectador: ");
   get(cinepreu_esp);new_line;
   crear_cinema(cine, cinenom, cinesales, cinecapacitat, cine_diaesp, cinepreu, cinepreu_cs, cinepreu_esp);

   exception
      when Ada.Text_IO.data_error => put_line(">> Valor incorrecte!!");
                                     skip_line;
                                     inicialitzar(fitxerhistoric);
	when constraint_error => put_line(">> Valor fora de rang!!");
                               skip_line;
                               inicialitzar(fitxerhistoric);
      when s_io.name_error => put_line("ERROR: noms d'arxius incorrectes");
	when s_io.use_error => put_line("ERROR: no hi ha permis per fer us dels arxius");
	when s_io.device_error => put_line("ERROR: el dispositiu no es troba en bon estat");

end inicialitzar;



--------------------------------------------------------------------------
-- Procediment que actualitza la cartellera des del fitxerpelis         --
-- i guarda un historial de les pelicules antigues al fitxerhistoric.   --
--------------------------------------------------------------------------
procedure actualitzar(fitxerpelis, fitxerhistoric: in string) is

pelis: Ada.Text_IO.file_type;
fitxerhist: s_io.file_type;

begin
   open(pelis, in_file, fitxerpelis);  -- Fitxer que conte les pelicules de la setmana
   open(fitxerhist, append_file, fitxerhistoric);  -- Fitxer historic
   posar_cartellera(pelis, cine);
   -- Ara comprovem els canvis a la cartellera i guardem les pelicules antigues
   for i in 1..quantes_sales(cine) loop
      -- El primer cop que s'agafen les pelis el nome es null, inicialitzem
      if (historic(i).pelicula="null                                                                                                                                                                                                    ") then
         historic(i).pelicula:=quina_peli(cine,i);
         historic(i).setmanes:=1;
      -- Si la pelicula nova es diferent a la antiga, guardem al fitxer historic i
      -- actualitzem la variable historic amb la nova pelicula
      elsif (quina_peli(cine,i)/=historic(i).pelicula) then
         write(fitxerhist, historic(i));
         historic(i).pelicula:=quina_peli(cine,i);
         historic(i).espectadors:=0;
         historic(i).setmanes:=1;
      -- sino incrementem el numero de setmanes que porta en cartellera
      else
         historic(i).setmanes:=historic(i).setmanes+1;
      end if;
   end loop;
   close(fitxerhist);
   close(pelis);

   exception
      when s_io.name_error => put_line("ERROR: noms d'arxius incorrectes");
	when s_io.use_error => put_line("ERROR: no hi ha permis per fer us dels arxius");
	when s_io.device_error => put_line("ERROR: el dispositiu no es troba en bon estat");

end actualitzar;



--------------------------------------------------------------------------
-- Procediment que realitza una consulta de quines pelicules d'un cert  --
-- genere estan a cartellera (mostra els titols)                        --
--------------------------------------------------------------------------
procedure quines_pelis_genere is

pausa: natural; -- Per fer pauses a les visualitzacion llarges
pausastring: string(1..20);
pausalong: integer;
generepeli: genere;

begin
   pausa:=0;
   put("Genere: "); get(generepeli);new_line;
   -- Fem un recorregut per totes les pelicules actuals i mostrem el titol
   -- d'aquelles que conincideixen amb el genere demanat
   for i in 1..quantes_sales(cine) loop
      if (pausa>20) then   -- Per una visualitzacio correcte, cada x linies s'aturara la pantalla
         put("Pressioni ENTER per continuar...");
         skip_line;
         get_line(pausastring, pausalong);
         pausa:=0;
      end if;
      if (generepeli=quin_genere (cine, i)) then
         pausa:=pausa+2;
         put("Sala: "); put(i); new_line;
         put("Pelicula: ");put(quina_peli(cine, i)(1..69));new_line;
      end if;
   end loop;
   put("[Fi] Pressioni ENTER...");
   skip_line;
   get_line(pausastring, pausalong);

   exception
      when Ada.Text_IO.data_error => put_line(">> Valor incorrecte!!");
                                     skip_line;
                                     quines_pelis_genere;
	when constraint_error => put_line(">> Valor fora de rang!!");
                               skip_line;
                               quines_pelis_genere;

end quines_pelis_genere;



--------------------------------------------------------------------------
-- Procediment que consulta quines pelicules es projecten, mostrant la  --
-- informacio de cada pelicula (titol, durada i genere)                 --
--------------------------------------------------------------------------
procedure quines_pelis is

pausa: natural; -- Per fer pauses a les visualitzacion llarges
pausastring: string(1..20);
pausalong: integer;
generepeli: genere;
pelitemps: temps;

begin
   pausa:=0;
   -- Fem un recorregut i mostrem totes les pelicules i les seves caracteristiques
   for i in 1..quantes_sales(cine) loop
      pausa:=pausa+5;
      if (pausa>20) then   -- Per una visualitzacio correcte, cada x linies s'aturara la pantalla
         put("Pressioni ENTER per continuar...");
         get_line(pausastring, pausalong);
         pausa:=0;
      end if;
      put("-------------------------------------------------------------------------------");new_line;
      put("Sala "); put(i, width=>2); new_line;
      put("Pelicula: "); put_line(quina_peli(cine, i)(1..69));
      pelitemps:=durada (cine, i);
      put("Durada: "); put(pelitemps, width=>4); new_line;
      generepeli:=quin_genere (cine, i);
      put("Genere: "); put(generepeli); new_line;
   end loop;
   put("[Fi] Pressioni ENTER...");
   get_line(pausastring, pausalong);

   exception
      when Ada.Text_IO.data_error => put_line(">> Valor incorrecte!!");
                                     skip_line;
	when constraint_error => put_line(">> Valor fora de rang!!");
                               skip_line;

end quines_pelis;



--------------------------------------------------------------------------
-- Procediment que consulta quins dies es pot veure una pelicula per    --
-- menys de x ptes.                                                     --
--------------------------------------------------------------------------
procedure quins_dies is

pesetes: natural;

begin
   put("Maxim preu: "); get(pesetes);
   put("Dies..."); new_line;
   -- Fem un recorregut per tots els dies i imprimim aquell que surt mes economic
   for i in dilluns..diumenge loop
      if (pesetes>=quant_costa(cine, i)) then
         put(i);new_line;
      end if;
   end loop;
   put("Fi llistat."); new_line;

   exception
      when Ada.Text_IO.data_error => put_line(">> Valor incorrecte!!");
                                     skip_line;
                                     quins_dies;
	when constraint_error => put_line(">> Valor fora de rang!!");
                               skip_line;
                               quins_dies;
end quins_dies;



--------------------------------------------------------------------------
-- Procediment que compra una entrada en una butaca concreta d'una certa--
-- sala i cinema.                                                       --
--------------------------------------------------------------------------
-- Com que la implementacio de veure_finestra esta mes abaix i la necessitem crida
-- en comprar, posem la seva capalera aqui.
procedure veure_finestra(cinesala: in num_sales; cinesessio: in sessions);

procedure comprar is

cinesala: num_sales;
cinebutaca: positive;
cinesessio: sessions;
nosala: exception;

begin
   -- Demanem les dades necessaries
   put("Sala: "); get(cinesala);
   put("Sessio: "); get(cinesessio);
   veure_finestra(cinesala, cinesessio);
   put("Butaca: "); get(cinebutaca);
   destroy_graph_window;
   if (cinesala>quantes_sales(cine)) then   -- Si no existeix la sala
      raise nosala;
   elsif (esta_tot_ple (cine, cinesala, cinesessio)) then   -- Si no existeix la sala
      put("Aquesta sala i sessio esta plena."); new_line;
   elsif (esta_lliure(cine, cinesala, cinesessio, cinebutaca)) then
      ocupar(cine, cinesala, cinesessio, cinebutaca);  -- Si la butca no existeix ho te en compte la funcio
      -- incrementem el nombre d'espectador a l'historic
      historic(cinesala).espectadors:=historic(cinesala).espectadors+1;
      put("Lliure... OK!"); new_line;
   else
      put("Butaca ocupada!"); new_line;
   end if;

   exception
      when nosala => put("No existeix la sala.");new_line;
                     skip_line;
                     comprar;
      when Ada.Text_IO.data_error => put_line(">> Valor incorrecte!!");
                                     skip_line;
                                     comprar;
	when constraint_error => put_line(">> Valor fora de rang!!");
                               skip_line;
                               comprar;
end comprar;



--------------------------------------------------------------------------
-- Procediment que simula la compra de 25 entrades de forma aleatoria
--------------------------------------------------------------------------
procedure compra_aleatoria is

sessioaleatori: sessioal;
salaaleatori: salaal;
butacaaleatori: butacaal;
gen_sessio: dr1.generator;  -- Per la generacio de num. aleatoris
gen_sala: dr2.generator;  -- Per la generacio de num. aleatoris
gen_butaca: dr3.generator;  -- Per la generacio de num. aleatoris

begin
   put("Generant...");new_line;
   reset(gen_sessio); -- Preparar el sistema per a generar numeros
   reset(gen_sala); -- Preparar el sistema per a generar numeros
   reset(gen_butaca); -- Preparar el sistema per a generar numeros
   for i in 1..25 loop
      loop
         sessioaleatori:=random(gen_sessio); -- generar numero aleatori
         sessioaleatori:=random(gen_sessio); -- generar numero aleatori
         salaaleatori:=random(gen_sala); -- generar numero aleatori
         -- Mentre que no ens doni una sala i sessio buida i una sala que existeixi
         while ((esta_tot_ple(cine, salaaleatori, sessioaleatori)) or (salaaleatori>quantes_sales(cine))) loop
            sessioaleatori:=random(gen_sessio); -- generar numero aleatori
            salaaleatori:=random(gen_sala); -- generar numero aleatori
         end loop;
         butacaaleatori:=random(gen_butaca); -- generar numero aleatori
         -- Mentre la butaca sigui mes gran que el numero real de butaques
         while ((butacaaleatori>mirar_capacitat(cine))) loop
            butacaaleatori:=random(gen_butaca); -- generar numero aleatori
         end loop;
         -- Sortim si finalment la sala, sessio i butaca es correcte i esta lliure
         exit when (esta_lliure(cine, salaaleatori, sessioaleatori, butacaaleatori)=true);
       end loop;
       ocupar(cine, salaaleatori, sessioaleatori, butacaaleatori);
       -- incrementem el nombre d'espectador a l'historic
       historic(salaaleatori).espectadors:=historic(salaaleatori).espectadors+1;
    end loop;
    put("Fi.");new_line;
end compra_aleatoria;



--------------------------------------------------------------------------
-- Procediment que mostra per pantalla les pelicules que ja han sigut   --
-- projectades (guardades al fitxerhistoric).                           --
--------------------------------------------------------------------------
procedure veure_historic(fitxerhistoric: in string) is

pausa: natural; -- Per fer pauses a les visualitzacion llarges
pausastring: string(1..20);
pausalong: integer;
visualitzarhist: historictype;  -- Per visualitzar el fitxer sequencial historic
fitxerhist: s_io.file_type;

begin
   open(fitxerhist, in_file, fitxerhistoric);
   pausa:=0;
   while not end_of_file(fitxerhist) loop
      pausa:=pausa+4;
      if (pausa>20) then   -- Per una visualitzacio correcte, cada x linies s'aturara la pantalla
         put("Pressioni ENTER per continuar...");
         get_line(pausastring, pausalong);
         pausa:=0;
      end if;
      read(fitxerhist, visualitzarhist);
      put("-------------------------------------------------------------------------------");new_line;
      put("Pelicula: ");put(visualitzarhist.pelicula(1..69));new_line;
      put("Espectadors: ");put(visualitzarhist.espectadors);new_line;
      put("Setmanes en cartellera: ");put(visualitzarhist.setmanes);new_line;
   end loop;
   put("[Fi] Pressioni ENTER...");
   get_line(pausastring, pausalong);
   close(fitxerhist);

   exception
      when s_io.name_error => put_line("ERROR: noms d'arxius incorrectes");
	when s_io.use_error => put_line("ERROR: no hi ha permis per fer us dels arxius");
	when s_io.device_error => put_line("ERROR: el dispositiu no es troba en bon estat");

end veure_historic;



--------------------------------------------------------------------------
-- Procediment que mostra una finestra amb totes les butaques ocupades  --
-- (vermell) i lluires (verd) de la sala i sessio indicada.             --
--------------------------------------------------------------------------
procedure veure_finestra(cinesala: in num_sales; cinesessio: in sessions) is

-- Windolin
xmax, ymax, xchar, ychar: integer;
butacacolor: color_type;
centrey, letras: integer;
nosala: exception;

begin
   -- Si no existeix la sala
   if (cinesala>quantes_sales(cine)) then
      raise nosala;
   else
      -- Si ja esta oberta la finestra, la tanquem
      if (is_open) then
         destroy_graph_window;
      end if;
      -- Creacio de la finestra i del titol
      create_graph_window(xmax, ymax, xchar, ychar);
      set_window_title(mirar_nom(cine)(1..lnom));
      -- Capa a on estara el titol
      draw_box(10,50,xmax-10,0,white,fill);
      letras:=(xmax-100)/xchar;  -- Numero de letras que hi caben a la capsa
      centrey:=25-(ychar/2);   -- Centrem el titol en vertical
      display_text(50, centrey, quina_peli(cine, cinesala)(1..letras), black);
      -- Imprimim butaques per files (15 files x 10 columnes = 150 butaques maximes)
      for j in 0..14 loop
         for i in 0..9 loop
            if (mirar_capacitat(cine)<1+i+(j*10)) then
               -- Si la butaca corresponent surt del rang de nombre de butaques que te el cinema
               -- actual, la pintem de negre per ferla invisible
               butacacolor:=black;
            elsif (esta_lliure(cine, cinesala, cinesessio, 1+i+(j*10))) then
               -- Si la butaca esta lliure la pintem de verd
               butacacolor:=light_green;
            else
               -- Si la butaca esta ocupada la pintem de vermell
               butacacolor:=light_red;
            end if;
            -- Dibuixem la butaca
            draw_box(10+i*((xmax-20)/10),  ymax-j*((ymax-50)/15),  10+i*((xmax-20)/10)+(xmax-20)/10, ymax-j*((ymax-50)/15)-(ymax-50)/15, butacacolor, fill);
            -- Dibuixem el perfil de la botaca de color negre per veure-les separades per 1 pixel
            draw_box(10+i*((xmax-20)/10),  ymax-j*((ymax-50)/15),  10+i*((xmax-20)/10)+(xmax-20)/10, ymax-j*((ymax-50)/15)-(ymax-50)/15, black, no_fill);
         end loop;
       end loop;
    end if;

   exception
      when nosala => put("No existeix la sala."); new_line;
                     skip_line;
      when Ada.Text_IO.data_error => put_line(">> Valor incorrecte!!");
                                     skip_line;
	when constraint_error => put_line(">> Valor fora de rang!!");
                               skip_line;
      when DLL_version_error => put_line(">> Versio incorrecta de DLL!!");
      when Invalid_coordinate => put_line(">> Coordenades invalides!!");
      when Window_Not_Open => put_line(">> Finestra no oberta!!");
      when Window_Already_Open => put_line(">> Finestra ja oberta!!");
      when Window_Already_closed => put_line(">> Finestra ja tancada!!");

end veure_finestra;


cinesala: num_sales;
cinesessio: sessions;

-- <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> --
-- Programa Principal                                               --
-- <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> --
begin
  inicialitzar("espectadors.dat");  -- Demanem totes les dades per crear el cinema
  put_line("ATENCIO: Per comenar a treballar primer ");
  put_line("ha d'actualitzar la cartellera (opcio a) per evitar errors");new_line;
  while op/=s loop
     put("CINEMES: ");put_line(mirar_nom(cine)(1..lnom));new_line;
     put("a. Actualitzar la cartellera"); new_line;
     put("b. Consultar quines pelicules d'un cert genere estan a cartellera"); new_line;
     put("c. Consultar quines pelicules es projecten"); new_line;
     put("d. Consultar quins dies puc veure una pelicula per menys de x pts"); new_line;
     put("e. Comprar una entrada"); new_line;
     put("f. Simular la compra de 25 entrades de forma aleatoria"); new_line;
     put("g. Visualitzar historic");new_line;
     put("h. Finestra d'una sala i sessio");new_line;
     put("i. Tancar finestra");new_line;
     put("j. Buidar sales (Final del dia)");new_line;
     put("s. SORTIR"); new_line;
     put("-> ");
     get(op);
     skip_line;
     case op is
       when a => actualitzar("pelis-cat.txt","espectadors.dat");
       when b => quines_pelis_genere;
       when c => quines_pelis;
       when d => quins_dies;
       when e => comprar;
       when f => compra_aleatoria;
       when g => veure_historic("espectadors.dat");
       when h => put("Sala: "); get(cinesala);
                 put("Sessio: "); get(cinesessio);
                 veure_finestra(cinesala, cinesessio);
       when i => if (is_open) then 
                    destroy_graph_window;
                 end if;
       when j => buidar_sales(cine);
       when others => null;
     end case;
     new_line;
  end loop;

  exception
      when Ada.Text_IO.data_error => put_line(">> Opcio incorrecte!!");
                                     skip_line;
	when constraint_error => put_line(">> Opcio fora de rang!!");
                               skip_line;
      when DLL_version_error => put_line(">> Versio incorrecta de DLL!!");
      when Invalid_coordinate => put_line(">> Coordenades invalides!!");
      when Window_Not_Open => put_line(">> Finestra no oberta!!");
      when Window_Already_Open => put_line(">> Finestra ja oberta!!");
      when Window_Already_closed => put_line(">> Finestra ja tancada!!");

end cine;